'use strict';

var config = require('../../config.json');

var DEFAULT_COMMENT_TOP_PADDING = -1;
var ALLOWED_COMMENT_TYPE = 'CommentBlock';

/**
 * @param {PluginPass} state
 * @returns {String|Boolean}
 */
function getDirective(state) {
    var directive = void 0;

    if ('useDirective' in state.opts && state.opts.useDirective !== true) {
        directive = state.opts.useDirective;
    } else {
        directive = config.default.useDirective;
    }

    return directive;
}

/**
 * @param {NodePath} path
 * @param {PluginPass} state
 * @param {Boolean} hasGlobalDirective
 * @returns {String|null}
 */
module.exports = function (path, state, hasGlobalDirective) {
    var node = path.node;

    if (!node.leadingComments) {
        return null;
    }

    var comments = node.leadingComments;

    var previousNode = path.key !== 0 ? path.getSibling(path.key - 1) : null;
    var previousNodeEnd = void 0;

    if (previousNode && previousNode.node) {
        previousNodeEnd = previousNode.node.end || DEFAULT_COMMENT_TOP_PADDING;
    } else {
        previousNodeEnd = DEFAULT_COMMENT_TOP_PADDING;
    }

    var functionDeclarationStart = void 0;

    /**
     * Fix for annoying edge case, when ObjectProperty node doesn't have start index
     * Workaround: find function body and get it's start index.
     */
    if (path.isObjectProperty()) {
        functionDeclarationStart = node.start || path.get('value.body').node.start;
    } else {
        functionDeclarationStart = node.start;
    }

    var foundedComment = void 0;

    for (var index = comments.length - 1, comment; index >= 0; index--) {
        comment = comments[index];

        if (comment.type === ALLOWED_COMMENT_TYPE && comment.start > previousNodeEnd && comment.end < functionDeclarationStart) {
            foundedComment = comment;
            break;
        }
    }

    if (!foundedComment) {
        return null;
    }

    var directive = getDirective(state);

    if (typeof directive === 'string' && !hasGlobalDirective) {
        foundedComment = foundedComment.value.includes('@' + directive) ? foundedComment : null;
    }

    return foundedComment ? foundedComment.value : null;
};